local mod = ...



-- *************************************************************************************************************
-- Ruin component -------------------------------------------------------------------------------------
-- A non-functioning component that is used to track ruins for the Fire mod and for proximity
-- *************************************************************************************************************

local COMP_RUIN = {
	TypeName = "COMP_RUIN",
	ParentType = "COMPONENT",
	Properties = {
        {
            Name = "InitFlag",
            Type = "boolean",
            Default = false,
            Flags = { "SAVE_GAME" },
        },
        {
            Name = "PrefabName",
            Type = "string",
            Default = nil,
            Flags = { "SAVE_GAME" },
        },
        {
            Name = "StepCount",
            Type = "integer",
            Default = 0,
            Flags = { "SAVE_GAME" },
        },
        {
            Name = "SpawnedObject",
            Type = "string",
            Default = nil,
            Flags = { "SAVE_GAME" },
        },
        {
            Name = "RotationMode",
            Type = "integer",
            Default = 0,
            Flags = { "SAVE_GAME" },
        },
        {
            Name = "Rotation",
            Type = "float",
            Default = 0.0,
            Flags = { "SAVE_GAME" },
        },
    },
}


-- Next piece
function COMP_RUIN:nextPiece()

    -- mod:log("Current step count: " .. self.StepCount)

    -- Decrement step count
    self.StepCount = self.StepCount - 1

    -- Delete the current visual piece and replace it with the next piece
    local objPieceCurrent = self:getLevel():find(self.SpawnedObject)
    if objPieceCurrent then

        -- Destroy the current piece
        objPieceCurrent:destroy()

    end

    -- Place the next piece
    if self.StepCount > 0 then

        local objSpawned = self:getLevel():createObject(self.PrefabName .. "_PIECE_" .. string.sub("000" .. self.StepCount, -3), self:getOwner():getGlobalPosition())
        if objSpawned then
            self.SpawnedObject = objSpawned:getId()
            objSpawned:rotateLocalY(self.Rotation)
        end

    -- Self destruct
    else

        -- Destroy the ruin itself
        self:getOwner():destroy()

    end

end



-- Register custom Class
mod:registerClass(COMP_RUIN)



-- *************************************************************************************************************
-- Ruin Gather component -------------------------------------------------------------------------------------
-- A part of the Ruin that is removed by salvage workers
-- *************************************************************************************************************

local COMP_RUIN_GATHER = {
	TypeName = "COMP_RUIN_GATHER",
	ParentType = "COMPONENT",
	Properties = {
    }
}

-- On Destroyed
function COMP_RUIN_GATHER:onDestroy(isClearingLevel)

    if not isClearingLevel then -- don't carry out this code if the game is shutting down, only if the ruin is removed during the game

        local compRuin = self:getOwner():findFirstObjectWithComponentUp("COMP_RUIN")

        if compRuin then
            compRuin:nextPiece()
        end

    end

end


-- Register custom Class
mod:registerClass(COMP_RUIN_GATHER)



-- Building Asset Processor -------------------------------------------------------------------------------------
mod:registerAssetProcessor("models/Ruin1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Register Ruins -------------------------------------------------------------------------------------
local function registerPrefab(mesh, name, buildingZone, startPiece, endPiece, startStone, endStone)

    -- Prefab
    mod:registerAssetId(mesh, name)

    -- Building Zone
    mod:registerPrefabComponent( name, {
        DataType = "COMP_BUILDING_ZONE",
        BuildingZone = buildingZone,
    })

    -- Ground
    mod:registerPrefabComponent( name, { DataType = "COMP_GROUNDED", AutoDisable = true, })

    -- Ruin component
    mod:registerPrefabComponent( name, { DataType = "COMP_RUIN", PrefabName = name, StepCount = endPiece - startPiece + 1 } )

    -- Loop through pieces
    for i = startPiece, endPiece do

        -- Piece prefab
        mod:registerAssetId( mesh .. ".Piece/Piece." .. string.sub("000" .. i, -3), name .. "_PIECE_" .. string.sub("000" .. i - startPiece + 1, -3))

        -- Gather position component
        mod:registerPrefabComponent( mesh .. "/Gather." .. string.sub("000" .. i, -3), { DataType = "COMP_RUIN_GATHER" } )

        -- Resource container
        mod:registerPrefabComponent( mesh .. "/Gather." .. string.sub("000" .. i, -3), {
            DataType = "COMP_RESOURCE_CONTAINER",
            ResourceData = "RESOURCE_RUIN",
            IsReplenishable = false,
            ResourceValue = 1.0, -- how many are gathered with each pick
            AvailableQuantity = 5, -- maximum container stock
            Radius = 1.0, -- distance from container that the villager will stand to pick
            IsDestroyWhenEmpty = true,
        })

    end

    -- Stones
    for j = startStone, endStone do

        mod:registerPrefabComponent( mesh .. "/Stone." .. string.sub("000" .. j, -3), { DataType = "COMP_GROUNDED", AutoDisable = true, })

    end

end


-- Ruin 1 -- large round tower
local zoneRuin1 = {
    ZoneEntryList = {
        {
            Polygon = polygon.createCircle( 5.5, {0.0, 0.0}, 16 ),
            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false }
        },
    },
}

registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.001", "PREFAB_RUIN_RUIN1", zoneRuin1, 1, 13, 1, 12)


-- Ruin 2 -- small church
local zoneRuin2 = {
    ZoneEntryList = {
        {
            Polygon = polygon.createRectangle( {12, 7}, {0.0, 0.0} ),
            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false }
        },
    },
}

registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.002", "PREFAB_RUIN_RUIN2", zoneRuin2, 14, 24, 13, 21)


-- Ruin 3, 4, 5, 6 -- wall
local zoneRuin3 = {
    ZoneEntryList = {
        {
            Polygon = polygon.createRectangle( {5.1, 1.2}, {0.0, 0.0} ),
            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false }
        },
    },
}

registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.003", "PREFAB_RUIN_RUIN3", zoneRuin3, 25, 26, 22, 23)
registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.004", "PREFAB_RUIN_RUIN4", zoneRuin3, 27, 28, 24, 24)
registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.005", "PREFAB_RUIN_RUIN5", zoneRuin3, 29, 30, 25, 25)
registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.006", "PREFAB_RUIN_RUIN6", zoneRuin3, 31, 35, 26, 27)


-- Ruin 7 - Coliseum (not used for now)
registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.007", "PREFAB_RUIN_RUIN7", zoneRuin3, 36, 38, 28, 28)


-- Ruin 8 - Square tower
local zoneRuin8 = {
    ZoneEntryList = {
        {
            Polygon = polygon.createRectangle( {7, 7}, {0.0, 0.0} ),
            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = false }
        },
    },
}

registerPrefab("models/Ruin1.fbx/Prefab/Ruin.Part.008", "PREFAB_RUIN_RUIN8", zoneRuin8, 39, 48, 29, 32)



-- Ruin Collections -------------------------------------------------------------------------------------
mod:registerAssetId("models/Ruin1.fbx/Prefab/Collection.Part.001", "PREFAB_RUIN_COLLECTION1")
mod:registerAssetId("models/Ruin1.fbx/Prefab/Collection.Part.002", "PREFAB_RUIN_COLLECTION2")
mod:registerAssetId("models/Ruin1.fbx/Prefab/Collection.Part.003", "PREFAB_RUIN_COLLECTION3")
mod:registerAssetId("models/Ruin1.fbx/Prefab/Collection.Part.004", "PREFAB_RUIN_COLLECTION4")
mod:registerAssetId("models/Ruin1.fbx/Prefab/Collection.Part.005", "PREFAB_RUIN_COLLECTION5")
mod:registerAssetId("models/Ruin1.fbx/Prefab/Collection.Part.006", "PREFAB_RUIN_COLLECTION6")



-- -- Material -------------------------------------------------------------------------------------
-- mod:registerAssetId("models/Ruin1.fbx/Materials/Material.Vine1", "MATERIAL_RUIN_VINE1")
-- mod:overrideAsset({ Id = "MATERIAL_RUIN_VINE1", HasAlphaTest = true, })



-- Resource: Ruins -------------------------------------------------------------------------------------
mod:registerEnumValue( "RESOURCE_TYPE", "RESOURCE_TYPE_RUIN" )

mod:registerAsset({
	DataType = "RESOURCE",
	Id = "RESOURCE_RUIN",
	ResourceName = "RESOURCE_RUIN_NAME",
	Icon = "ICON_RESOURCE_STONE",
	ResourceTypeList = { "RESOURCE_TYPE_RUIN" },
	IsTradable = false,
    DisplayInInventory = false,
    DisplayInToolbar = false,
	ResourceLayoutType = "CRATES",
    OrderId = 0,
})



-- Building: Salvage Yard -------------------------------------------------------------------------------------

-- Building
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_RUIN_SALVAGE",
	Name = "BUILDING_RUIN_SALVAGE_NAME",
	Description = "BUILDING_RUIN_SALVAGE_DESC",
	BuildingType = "RESOURCE_PRODUCTION",
    AssetCoreBuildingPart = "BUILDING_PART_RUIN_SALVAGE",
    VillagerRequired = {
        Status = "NEWCOMER",
        Quantity = 10,
    },
})

-- Building Part
for i = 1, 4 do

    mod:registerAssetId("models/Ruin1.fbx/Prefab/Salvage.Part.00" .. i, "PREFAB_RUIN_SALVAGE" .. i)

    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_RUIN_SALVAGE" .. i,
        Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
        AssetBuildingFunction = "BUILDING_FUNCTION_RUIN_SALVAGE" .. i,
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_RUIN_SALVAGE" .. i,
        },
    })

    -- mod:registerPrefabComponent("PREFAB_RUIN_SALVAGE" .. i, {
    --     DataType = "COMP_RESOURCE_DEPOT",
    --     Capacity = 50,
    --     OutgoingReservedResources = {},
    --     IncomingReservedResources = {},
    -- })

end

-- Crate visuals
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE",
    Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_RESOURCE_VISUAL_STONE_CRATE",
    },
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_RUIN_SALVAGE_CRATE_GOLD",
    Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_RESOURCE_VISUAL_STONE",
    },
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_RUIN_SALVAGE_CRATE_IRON",
    Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_RESOURCE_VISUAL_IRON_BAR_CRATE",
    },
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_RUIN_SALVAGE_CRATE_MARBLE",
    Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_RESOURCE_VISUAL_MARBLE",
    },
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_RUIN_SALVAGE_CRATE_JEWELRY",
    Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_RESOURCE_VISUAL_CRATE",
    },
})


-- -- Storage function
-- mod:registerAsset({
--     DataType = "BUILDING_FUNCTION_RESOURCE_STOCKPILE",
--     Id = "BUILDING_FUNCTION_RUIN_STORAGE",
--     ResourceLayoutTypeList = {
--         RESOURCE_LAYOUT_TYPE.CRATES,
--     },
--     AllowedResourceList = {
--         "STONE",
--         "MARBLE",
--         "IRON",
--         "GOLD_BARS",
--         "JEWELRY",
--     },
--     VisualLocatorPrefix = "StorageLocator",
-- })

-- Assemblage
mod:registerAssetId("models/Ruin1.fbx/Prefab/Salvage.Assemblage.Part", "PREFAB_RUIN_SALVAGE_ASSEMBLAGE")
mod:registerAssetId("models/Ruin1.fbx/Prefab/Salvage.Assemblage.Part.Constr", "PREFAB_RUIN_SALVAGE_ASSEMBLAGE_CONSTR")

for i = 1, 20 do
    mod:registerPrefabComponent("models/Ruin1.fbx/Prefab/Salvage.Assemblage.Part/Attach_Minor." .. string.sub("000" .. i, -3), { DataType = "COMP_GROUNDED" })
end


-- mod:registerPrefabComponent("PREFAB_RUIN_SALVAGE_ASSEMBLAGE", {
--     DataType = "COMP_RESOURCE_STOCKPILE",
--     ResourceLayoutTypeList = {
--         RESOURCE_LAYOUT_TYPE.CRATES,
--     },
--     OptionalAllowedResourceList = {
--         "STONE",
--         "MARBLE",
--         "IRON",
--         "GOLD_BARS",
--         "JEWELRY",
--     },
--     DefaultResourceVisual = "PREFAB_RESOURCE_VISUAL_CRATE",
--     LocatorPrefix = "StorageLocator",
-- })

mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_RUIN_SALVAGE_ASSEMBLAGE",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		CoreRandomBuildingPartList = { "PREFAB_RUIN_SALVAGE_ASSEMBLAGE" },
		MandatoryBuildingPartList = {
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE1", OptionalAttachNodeString = "Attach_Major.001" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE2", OptionalAttachNodeString = "Attach_Major.002" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE3", OptionalAttachNodeString = "Attach_Major.003" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE4", OptionalAttachNodeString = "Attach_Major.004" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.001" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.002" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_IRON", OptionalAttachNodeString = "Attach_Minor.003" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.004" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.005" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.006" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_JEWELRY", OptionalAttachNodeString = "Attach_Minor.007" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.008" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.009" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_MARBLE", OptionalAttachNodeString = "Attach_Minor.010" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.011" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.012" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.013" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_GOLD", OptionalAttachNodeString = "Attach_Minor.014" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.015" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_STONE", OptionalAttachNodeString = "Attach_Minor.016" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_GOLD", OptionalAttachNodeString = "Attach_Minor.017" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_IRON", OptionalAttachNodeString = "Attach_Minor.018" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_MARBLE", OptionalAttachNodeString = "Attach_Minor.019" },
			{ BuildingPart = "BUILDING_PART_RUIN_SALVAGE_CRATE_JEWELRY", OptionalAttachNodeString = "Attach_Minor.020" },
		},
	},
    -- AssetBuildingFunction = "BUILDING_FUNCTION_RUIN_STORAGE",
})

-- Final Part
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_RUIN_SALVAGE",
    Name = "BUILDING_PART_RUIN_SALVAGE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_RUIN_SALVAGE_ASSEMBLAGE",
        },
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0.0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0.0,
            CameraPitchOffset = 0.0
        },
    },
    ConstructionVisual = "PREFAB_RUIN_SALVAGE_ASSEMBLAGE_CONSTR",
	BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createRectangle( {8.5, 8.5}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true },
			},
		}
	},
    Cost = {
        BuildRightTaxes = {
            { Resource = "GOLD_COINS", Quantity = 50 },
        },
        UpkeepCost = { 
            { Resource = "GOLD", Quantity = 10 },
        },
        ResourceNeededList = {
            {
                { Resource = "WOOD", Quantity = 8 },
                { Resource = "PLANK", Quantity = 10 },
                { Resource = "TOOL", Quantity = 4 },
                { Resource = "WOOLEN_CLOTH", Quantity = 20 },
            },
        },
    },
})



-- Job: Salvage Worker -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_RUIN_SALVAGE",
	JobName = "JOB_RUIN_SALVAGE_NAME",
	JobDescription = "JOB_RUIN_SALVAGE_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_RUIN_GATHER",
    ProductionDelay = 30, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
    RelatedZone = "EXTRACTION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
        RightHandObject = {
            Model = "PREFAB_TOOL_STONECUTTER_PICKAXE",
        },
		IdleAnimation = "MINE",
	},
})

mod:overrideAsset({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_RUIN_SALVAGE"
    }
})

mod:overrideAsset({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_RUIN_SALVAGE"
    }
})

mod:overrideAsset({
    Id = "COMMONER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_RUIN_SALVAGE"
    }
})



-- Building Function: Ruin Salvage -------------------------------------------------------------------------------------

-- mod:registerAsset({
--     DataType = "BUILDING_FUNCTION",
--     Id = "BUILDING_FUNCTION_RUIN_SALVAGE_MODIFIER",
--     Name = "BUILDING_FUNCTION_RUIN_SALVAGE_NAME",
--     GameRuleModifierList = {
--         DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
--         ResourceList = {
--             "RESOURCE_RUIN",
--         },
--         YieldPercentage = 0.25,
--         IsUntilRemoval = false,
--         IsGlobal = true,
--         DurationInDays = 56
--     },
-- })

local function registerSalvageFunction(index, resource1, resource2, chance)
    mod:registerAsset({
        DataType = "BUILDING_FUNCTION_QUARRY",
        Id = "BUILDING_FUNCTION_RUIN_SALVAGE" .. index,
        Name = "BUILDING_FUNCTION_RUIN_SALVAGE_NAME",
        WorkerCapacity = 1,
        RelatedJob = {
            Job = "JOB_RUIN_SALVAGE",
            Behavior = "BEHAVIOR_RUIN_GATHER",
        },
        UpkeepPerCapacity = {
            { Resource = "GOLD_COINS", Quantity = 2 },
        },
        InputInventoryCapacity = {
        },
        ResourceListNeeded = {
            { Resource = "RESOURCE_RUIN", Quantity = 1 },
        },
        ResourceProduced = {
            { Resource = resource1, Quantity = 1 },
        },
        RandomResourceToSpawn = resource2,
        SpawnPercentage = chance,
    })
end

registerSalvageFunction(1, "STONE", "IRON", 0.20)
registerSalvageFunction(2, "STONE", "MARBLE", 0.10)
registerSalvageFunction(3, "STONE", "JEWELRY", 0.05)
registerSalvageFunction(4, "STONE", "GOLD_BARS", 0.05)


